# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

LABEL2SURF_METADATA = Metadata(
    id="e79995ea6cf3a63ca3f02be2fa4a0665ff77a9de.boutiques",
    name="label2surf",
    package="fsl",
    container_image_tag="brainlife/fsl:6.0.4-patched2",
)


Label2surfParameters = typing.TypedDict('Label2surfParameters', {
    "__STYX_TYPE__": typing.Literal["label2surf"],
    "input_surface": InputPathType,
    "output_surface": str,
    "labels": InputPathType,
    "verbose_flag": bool,
    "help_flag": bool,
})


def dyn_cargs(
    t: str,
) -> typing.Any:
    """
    Get build cargs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build cargs function.
    """
    return {
        "label2surf": label2surf_cargs,
    }.get(t)


def dyn_outputs(
    t: str,
) -> typing.Any:
    """
    Get build outputs function by command type.
    
    Args:
        t: Command type.
    Returns:
        Build outputs function.
    """
    return {
        "label2surf": label2surf_outputs,
    }.get(t)


class Label2surfOutputs(typing.NamedTuple):
    """
    Output object returned when calling `label2surf(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    out_surf: OutputPathType
    """Resulting output surface file"""


def label2surf_params(
    input_surface: InputPathType,
    output_surface: str,
    labels: InputPathType,
    verbose_flag: bool = False,
    help_flag: bool = False,
) -> Label2surfParameters:
    """
    Build parameters.
    
    Args:
        input_surface: Input surface file.
        output_surface: Output surface file.
        labels: ASCII list of label files.
        verbose_flag: Switch on diagnostic messages.
        help_flag: Display help message.
    Returns:
        Parameter dictionary
    """
    params = {
        "__STYXTYPE__": "label2surf",
        "input_surface": input_surface,
        "output_surface": output_surface,
        "labels": labels,
        "verbose_flag": verbose_flag,
        "help_flag": help_flag,
    }
    return params


def label2surf_cargs(
    params: Label2surfParameters,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("label2surf")
    cargs.extend([
        "--surf",
        execution.input_file(params.get("input_surface"))
    ])
    cargs.extend([
        "--out",
        params.get("output_surface")
    ])
    cargs.extend([
        "--labels",
        execution.input_file(params.get("labels"))
    ])
    if params.get("verbose_flag"):
        cargs.append("--verbose")
    if params.get("help_flag"):
        cargs.append("--help")
    return cargs


def label2surf_outputs(
    params: Label2surfParameters,
    execution: Execution,
) -> Label2surfOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = Label2surfOutputs(
        root=execution.output_file("."),
        out_surf=execution.output_file(params.get("output_surface")),
    )
    return ret


def label2surf_execute(
    params: Label2surfParameters,
    execution: Execution,
) -> Label2surfOutputs:
    """
    Transform a group of labels into a surface.
    
    Author: FMRIB Analysis Group, University of Oxford
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki
    
    Args:
        params: The parameters.
        execution: The execution object.
    Returns:
        NamedTuple of outputs (described in `Label2surfOutputs`).
    """
    params = execution.params(params)
    cargs = label2surf_cargs(params, execution)
    ret = label2surf_outputs(params, execution)
    execution.run(cargs)
    return ret


def label2surf(
    input_surface: InputPathType,
    output_surface: str,
    labels: InputPathType,
    verbose_flag: bool = False,
    help_flag: bool = False,
    runner: Runner | None = None,
) -> Label2surfOutputs:
    """
    Transform a group of labels into a surface.
    
    Author: FMRIB Analysis Group, University of Oxford
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki
    
    Args:
        input_surface: Input surface file.
        output_surface: Output surface file.
        labels: ASCII list of label files.
        verbose_flag: Switch on diagnostic messages.
        help_flag: Display help message.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Label2surfOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(LABEL2SURF_METADATA)
    params = label2surf_params(
        input_surface=input_surface,
        output_surface=output_surface,
        labels=labels,
        verbose_flag=verbose_flag,
        help_flag=help_flag,
    )
    return label2surf_execute(params, execution)


__all__ = [
    "LABEL2SURF_METADATA",
    "Label2surfOutputs",
    "Label2surfParameters",
    "label2surf",
    "label2surf_params",
]
